@@ -24,6 +24,12 @@
 #include "config.h"
 
 #include "gnome-keyring.h"
+#include "gkr-misc.h"
+
+#include "mock-service.h"
+
+#include "egg/egg-testing.h"
+
 #include <glib.h>
 
 #include <stdlib.h>
@@ -41,27 +47,6 @@ static gchar *default_name = NULL;
 #define DISPLAY_NAME "Item Display Name"
 #define SECRET "item-secret"
 
-static GMainLoop *mainloop = NULL;
-
-static gboolean
-quit_loop (gpointer data)
-{
-	g_main_loop_quit (data);
-	return TRUE;
-}
-
-static void
-mainloop_run (int timeout)
-{
-	guint id = 0;
-
-	if (timeout)
-		id = g_timeout_add (timeout, quit_loop, mainloop);
-	g_main_loop_run (mainloop);
-	if (timeout)
-		g_source_remove (id);
-}
-
 static void
 test_remove_incomplete (void)
 {
@@ -406,7 +391,7 @@ static void
 done_grant_access (GnomeKeyringResult res, gpointer data)
 {
 	grant_access_result = res;
-	g_main_loop_quit (mainloop);
+	egg_test_wait_stop ();
 }
 
 static void
@@ -433,7 +418,7 @@ test_keyring_grant_access (void)
 	/* "callback already called" */
 	g_assert_cmpint (grant_access_result, ==, GNOME_KEYRING_RESULT_CANCELLED);
 
-	mainloop_run (2000);
+	egg_test_wait ();
 
 	g_assert_cmpint (GNOME_KEYRING_RESULT_OK, ==, grant_access_result);
 
@@ -461,7 +446,7 @@ static void
 done_store_password (GnomeKeyringResult res, gpointer data)
 {
 	*((GnomeKeyringResult*)data) = res;
-	g_main_loop_quit (mainloop);
+	egg_test_wait_stop ();
 }
 
 static void
@@ -497,7 +482,7 @@ test_store_password (void)
 	/* "callback already called" */
 	g_assert_cmpint (res, ==, GNOME_KEYRING_RESULT_CANCELLED);
 
-	mainloop_run (2000);
+	egg_test_wait ();
 
 	g_assert_cmpint (GNOME_KEYRING_RESULT_OK, ==, res);
 }
@@ -516,7 +501,7 @@ done_find_password (GnomeKeyringResult res, const gchar* password, gpointer unus
 		g_assert_cmpstr (password, ==, "password");
 	}
 
-	g_main_loop_quit (mainloop);
+	egg_test_wait_stop ();
 }
 
 static void
@@ -554,7 +539,7 @@ test_find_password (void)
 	/* "callback already called" */
 	g_assert (find_password_result == GNOME_KEYRING_RESULT_CANCELLED);
 
-	mainloop_run (2000);
+	egg_test_wait ();
 
 	g_assert_cmpint (GNOME_KEYRING_RESULT_OK, ==, find_password_result);
 }
@@ -566,7 +551,7 @@ done_find_no_password (GnomeKeyringResult res, const gchar* password, gpointer u
 {
 	find_no_password_result = res;
 	g_assert (password == NULL);
-	g_main_loop_quit (mainloop);
+	egg_test_wait_stop ();
 }
 
 static void
@@ -594,7 +579,7 @@ test_find_no_password (void)
 	g_assert (op != NULL);
 	g_assert (find_no_password_result == GNOME_KEYRING_RESULT_CANCELLED);
 
-	mainloop_run (2000);
+	egg_test_wait ();
 
 	g_assert_cmpint (GNOME_KEYRING_RESULT_NO_MATCH, ==, find_no_password_result);
 }
@@ -603,7 +588,7 @@ static void
 done_delete_password (GnomeKeyringResult res, gpointer data)
 {
 	*((GnomeKeyringResult*)data) = res;
-	g_main_loop_quit (mainloop);
+	egg_test_wait_stop ();
 }
 
 static void
@@ -634,7 +619,7 @@ test_delete_password (void)
 	/* "callback already called" */
 	g_assert (res == GNOME_KEYRING_RESULT_CANCELLED);
 
-	mainloop_run (2000);
+	egg_test_wait ();
 
 	/* Should have already been deleted by the second call above */
 	g_assert_cmpint (GNOME_KEYRING_RESULT_OK, ==, res);
@@ -668,94 +653,27 @@ test_setup_environment (void)
 	g_assert_cmpint (GNOME_KEYRING_RESULT_OK, ==, res);
 }
 
-static pid_t daemon_pid = 0;
-
-static gboolean
-daemon_start (void)
-{
-	GError *err = NULL;
-	gchar *args[5];
-	const gchar *path, *service, *address;
-	gchar *output = NULL;
-	gint exit_status = 1;
-	GError *error = NULL;
-
-	/* Need to have DBUS running */
-	address = g_getenv ("DBUS_SESSION_BUS_ADDRESS");
-	if (!address || !address[0]) {
-		g_printerr ("\nNo DBUS session available, skipping tests!\n\n");
-		return FALSE;
-	}
-
-	/* Check if gnome-keyring-daemon has testing enabled */
-	if (!g_spawn_command_line_sync (GNOME_KEYRING_DAEMON_PATH " --version",
-	                                &output, NULL, &exit_status, &error)) {
-		g_printerr ("Couldn't launch '%s': %s", GNOME_KEYRING_DAEMON_PATH,
-		            error->message);
-		return FALSE;
-	}
-
-	/* For some reason --version failed */
-	if (exit_status != 0 || strstr (output, "testing: enabled") == NULL) {
-		g_printerr ("Skipping tests since no testing enabled gnome-keyring-daemin is available\n");
-		g_free (output);
-		return FALSE;
-	}
-
-	g_free (output);
-
-	path = g_getenv ("GNOME_KEYRING_TEST_PATH");
-	if (path && !path[0])
-		path = NULL;
-
-	service = g_getenv ("GNOME_KEYRING_TEST_SERVICE");
-	if (service && !service[0])
-		service = NULL;
-
-	if (!path && !service) {
-		g_mkdir_with_parents ("/tmp/keyring-test/data", 0700);
-		g_setenv ("GNOME_KEYRING_TEST_PATH", "/tmp/keyring-test/data", TRUE);
-		g_setenv ("GNOME_KEYRING_TEST_SERVICE", "org.gnome.keyring.Test", TRUE);
-
-		g_printerr ("Starting gnome-keyring-daemon...\n");
-
-		args[0] = GNOME_KEYRING_DAEMON_PATH;
-		args[1] = "--foreground";
-		args[2] = "--control-directory";
-		args[3] = "/tmp/keyring-test";
-		args[4] = NULL;
-
-		if (!g_spawn_async (NULL, args, NULL, G_SPAWN_LEAVE_DESCRIPTORS_OPEN | G_SPAWN_DO_NOT_REAP_CHILD,
-							NULL, NULL, &daemon_pid, &err)) {
-			g_error ("couldn't start gnome-keyring-daemon for testing: %s",
-					 err && err->message ? err->message : "");
-			g_assert_not_reached ();
-		}
-
-		/* Let it startup properly */
-		sleep (2);
-	}
-
-	return TRUE;
-}
-
-static void
-daemon_stop (void)
-{
-	if (daemon_pid)
-		kill (daemon_pid, SIGTERM);
-	daemon_pid = 0;
-}
-
 int
 main (int argc, char **argv)
 {
+	const gchar *address;
+	GError *error = NULL;
+	const gchar *service;
 	int ret = 0;
 
 	g_test_init (&argc, &argv, NULL);
 	g_set_prgname ("test-keyrings");
 
-	mainloop = g_main_loop_new (NULL, FALSE);
+	/* Need to have DBUS running */
+	address = g_getenv ("DBUS_SESSION_BUS_ADDRESS");
+	if (!address || !address[0]) {
+		g_printerr ("\nNo DBUS session available, skipping tests.\n\n");
+		return 0;
+	}
+
+	service = g_getenv ("GNOME_KEYRING_TEST_SERVICE");
+	if (service && service[0])
+		service = NULL;
 
 	g_test_add_func ("/keyrings/remove-incomplete", test_remove_incomplete);
 	g_test_add_func ("/keyrings/create-keyring", test_create_keyring);
@@ -780,16 +698,18 @@ main (int argc, char **argv)
 	g_test_add_func ("/keyrings/set-display", test_set_display);
 	g_test_add_func ("/keyrings/setup-environment", test_setup_environment);
 
-	if (daemon_start ()) {
-		ret = g_test_run ();
+	if (service) {
+		g_printerr ("running tests against secret service: %s", service);
+		gkr_service_name = service;
 
-		if (default_name) {
-			gnome_keyring_set_default_keyring_sync (default_name);
-			g_free (default_name);
-		}
+	} else if (!mock_service_start ("mock-service-normal.py", &error)) {
+		g_printerr ("\nCouldn't start mock secret service: %s\n\n", error->message);
+		return 1;
 
-		daemon_stop ();
 	}
 
+	ret = egg_tests_run_with_loop ();
+	mock_service_stop ();
+
 	return ret;
 }
